#ifndef INCLUDE__Mainwindow_hxx_
#define INCLUDE__Mainwindow_hxx_

#include <memory>
#include <map>
#include <set>

#include <QTextEdit>
#include <QPushButton>
#include <QMainWindow>
#include <QListWidget>
#include <QStackedWidget>

#include <log4cxx/logger.h>

class Disassembler;
class Function;
class InformationManager;
class BasicBlockWidget;
class ScriptingDock;

class Mainwindow : public QMainWindow {
	Q_OBJECT
public:
	Mainwindow(InformationManager* mgr);

private:
	void addFunction(Function* fun);
	void setGlobalHotkeys();

	QTextEdit *textEdit;
	QPushButton *quitButton;
	QMenu *fileMenu;

	QTabWidget * tabwidget;
	QListWidget * listWidget;
	QStackedWidget * stackedWidget;
	QDockWidget * dockWidget;
	ScriptingDock * scripting;

	QAction *exitAction;
	QAction *openAction;
	QAction *saveAction;

	std::map<uint64_t, BasicBlockWidget*> blocks;
	std::map<QListWidgetItem*, QWidget*> objects_list;
	std::map<uint64_t, QListWidgetItem*> objects_list_by_address;
	std::set<Function*> functions;
	InformationManager* manager;

	log4cxx::LoggerPtr logger;
public Q_SLOTS:
	void switchMainPlaneToAddress(uint64_t);
	void requestNewFunctionByAddress(uint64_t address);
private Q_SLOTS:
	void quit();
	void open();
	void save();
	void switchMainPlane(int);
	void showListContextMenu(const QPoint&);
	void requestNewFunction();
	void renameFunction(QListWidgetItem * item);
};

#endif /* INCLUDE__Mainwindow_hxx_ */
